/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "fitsio.h"
#include "imaging_data.h"
#include "imaging_fdata.h"
#include "minrtsFrameData.h"

char** strSplit (char *string, int *nString) {
   char *blank = " ";
   char *oldString;
   int   nS, iS ;
   char *tok;
   char *stringCopy;
   char **output;
   *nString = 0;
   stringCopy = (char *)malloc(sizeof(char) * (1+strlen(string)));
   strcpy (stringCopy, string);
   oldString = stringCopy;
   tok = strtok(oldString, blank);
   if (tok == NULL) return NULL;
   oldString = NULL;
   for (nS = 0; tok != NULL; nS++) {
      tok = strtok(oldString, blank);
   }
   *nString = nS;
   output = (char **)malloc(nS * sizeof(char *));
   strcpy (stringCopy, string);
   oldString = stringCopy;
   blank = " ";
   for (iS=0; iS<nS; iS++) {
      tok = strtok(oldString, blank);
      oldString = NULL;
      output[iS] = (char *)malloc(sizeof(char) * (1+strlen(tok)));
      strcpy(output[iS], tok);
   }
   free(oldString);
   return output;
} 

void killFileList(char **fileList, int nFiles) {
   int iFile;
   for (iFile=0; iFile<nFiles;iFile++) {free(fileList[iFile]);}
   free(fileList);
   return;
} /* end of killFileList */

int main (int argc, char** argv) {

   char **fileList;
   int nFiles, iFile;
   int nRegion, iRegion;
   int maxstep;
   int i,ierr;
   long nRows, iRow, oRow;
   int nx, ny;
   long npix, ipix;
   short *inPix;
   float *maskPix;
   int isMask;
   int iKey;

   fitsfile *inFile;
   fitsfile *outFile = NULL;
   imaging_data_table_def *idata;
   imaging_fdata_table_def *odata;
   imaging_detector_table_def *inDetectorDef;
   imaging_detector_table_def *outDetectorDef;
   imaging_data_row *inData;
   imaging_fdata_row *outData;
   minrtsFrameData *maskdata;

   fileList = strSplit(argv[1], &nFiles);

   for (i=0; i<nFiles; i++) {printf("%s \n", fileList[i]);}
   ierr = 0;
/*
      get mask data
*/
   if (0 == strlen(argv[2])) isMask = (1==0); 
   else {
      maskdata = minrtsFrameDataReadFits(argv[2], &ierr);
      isMask = (ierr == 0);
   }
   if (isMask) printf("using maskfile %s", argv[2]); 
   else if (ierr !=0) 
      fprintf(stderr, "maskfile %s gave error %i",argv[2], ierr);

/*
      open first file and get detector info; then close file
*/
   fits_open_file (&inFile, fileList[0], READONLY, &ierr);
   if (ierr != 0) {
      printf("fits openfile of %s gave error %i",fileList[0],
         ierr);
      return 1;
   }
   outDetectorDef = read_imaging_detector_table(inFile, &ierr);
   idata          = open_imaging_data_table(inFile, &ierr);
   inData         = make_imaging_data_row(idata);
   nRegion        = idata->nregion;
   iKey = table_find_key(idata->def, "MAXSTEP"); 
   if (iKey>0)  table_get_keyword (idata->def, iKey, (void*)&maxstep, 
      &ierr);
   i = fits_close_file(inFile, &ierr);
/*
      reset output data sizes to reflect compression
*/
   for (iRegion=0; iRegion<nRegion; iRegion++) {
      outDetectorDef->naxis[iRegion][0] = 1;
      outDetectorDef->naxis[iRegion][1] = 1;
   }
/*
      create output data table 
*/
   fits_create_file (&outFile, argv[3], &ierr);
   if (ierr !=0) {
      fprintf (stderr, "error opening fits output %s %i\n",argv[3], ierr);
      return 1;
      }
   odata = make_imaging_fdata_table_def (outFile, 
      outDetectorDef, maxstep, &ierr); 
   if (ierr !=0) {
      fprintf (stderr, "error creating data definition\n");
      return 1;
      }
   outData = make_imaging_fdata_row(odata);
   oRow = 1;
/*   
    loop over input files
*/
   for (iFile=0; iFile<nFiles; iFile++) {
      fits_open_file (&inFile, fileList[iFile], READONLY, &ierr);
      if (ierr != 0) {
         printf("fits openfile of %s gave error %i",fileList[iFile],
            ierr);
         return 1;
      }
      inDetectorDef = read_imaging_detector_table(inFile, &ierr);
      idata = open_imaging_data_table(inFile, &ierr);
      i = fits_get_num_rows (inFile,  &nRows, &ierr); 
/* suppress autoscaling to avoid overflow */
      for (i=0; i<idata->nregion; i++) 
         fits_set_tscale(inFile, 1+idata->data_col[i], 1.0, 0.0, &ierr);
/*
         loop over input rows
*/
      for (iRow=1; iRow<=nRows; iRow++) {
         i = read_imaging_data_table(inFile, idata, inData, iRow, &ierr);
/*
            copy nonData info
*/
         outData->time = inData->time;
         outData->opd[0] = inData->opd[0];
         outData->opd[1] = inData->opd[1];
         outData->local_opd[1] = inData->local_opd[1];
         outData->local_opd[0] = inData->local_opd[0];
         outData->exp = inData->exp;
         outData->offset[0] = inData->offset[0];
         outData->offset[1] = inData->offset[1];
         outData->rotation  = inData->rotation;
         outData->frame     = inData->frame;
         outData->step      = inData->step;
         outData->reference = inData->reference;
         for (i=0; i<idata->nregion; i++) 
            outData->targtype[i] = inData->targtype[i];  
/*
            compress real data
*/
         for (iRegion=0; iRegion<nRegion; iRegion++)  {
            nx = inDetectorDef->naxis[iRegion][0];
            ny = inDetectorDef->naxis[iRegion][1];
            npix = nx * (long)ny;
            outData->data[iRegion][0] = 0.;
            inPix = (short *)inData->data[iRegion];
            if (isMask) {
               maskPix = maskdata->data[iRegion];
               for (ipix=0;ipix<npix;ipix++) 
                  outData->data[iRegion][0] =
                     outData->data[iRegion][0] +
               (*(inPix++)+32768.) * (*(maskPix++));
            } else for (ipix=0;ipix<npix;ipix++) 
               outData->data[iRegion][0] =
               outData->data[iRegion][0] + 32768.+
               *(inPix++);
         }
/*
            write output row
*/
         i = write_imaging_fdata_table(outFile, odata, outData, 
            oRow++, &ierr);
      }
      i = fits_close_file(inFile, &ierr);
   } /* end of loop over iFile */
   i = close_imaging_fdata_table(outFile, odata, &ierr);
   kill_imaging_fdata_row(outData);
   kill_imaging_data_row(inData);
   kill_imaging_data_table_def (idata,1);
   return 0;
}
